<?php

function check_edit($name)
{
  $sql = sprintf("SELECT sys_mask.id,edit FROM sys_mask "
		 . "JOIN sys_group_mask ON sys_mask.id = sys_group_mask.mask "
		 . "WHERE gid = %d AND fname = '%s' "
		 . "ORDER BY edit DESC LIMIT 1",
		 $_SESSION['sys']['group'], pg_escape_string($name));

  $sth = pg_query($sql);

  if ($sth === false) return false;

  $row = pg_fetch_assoc($sth);
  if ($row === false) return false;

  if ($row['edit'] == '0')
    return false;

  return true;
}

function build_form($name, $mask)
{
  global $jscode;
  $ret = array();

  $jscode[] = 'Rico.onLoad( function() {';
  $jscode[] = "form_init();";
  $jscode[] = '});';

  $ret[] = '<div class="form">';
  if (empty($mask['edit_title']))
    $ret[] = '<p class="title">Datensatz bearbeiten</p>';
  else
    $ret[] = sprintf('<p class="title">%s</p>', $mask['edit_title']);
  $ret[] = '<form id="form_edit">';

  $ret[] = '<input type="hidden" id="edit_id" name="id" value="">';
  $ret[] = sprintf('<input type="hidden" id="edit_source" name="source" value="%s">', $name);

  foreach ($mask['edit'] as $id => $info) {
    if ($info['type'] == 'hidden') {
      $ret[] = sprintf('<input type="hidden" id="edit_%s" name="%s">', $id, $id);
    } else if ($info['type'] == 'text' || $info['type'] == 'passwd' ||
	$info['type'] == 'decimal' || $info['type'] == 'number') {
      $v = array('id="edit_'.$id.'"',
		 'name="'.$id.'"');
      $v[] = 'size="'.(empty($info['size'])?'10':$info['size']).'"';
      $v[] = 'type="'.($info['type']=='passwd'?'password':'text').'"';
      $ret[] = sprintf('<label for="edit_%s">%s</label><br>', $id, $info['name']);
      $ret[] = sprintf('<input %s>', implode(' ', $v));
    } elseif ($info['type'] == 'date') {
      $ret[] = sprintf('<label for="edit_%s">%s</label><br>', $id, $info['name']);
      $ret[] = sprintf('<input id="edit_%s" name="%s" size="8">&nbsp;'.
		       '<img class="calendar" src="images/icons/calendar.gif" onclick="calendar(\'edit_%s\',event)" />',
		       $id, $id, $id);
    } elseif ($info['type'] == 'boolean') {
      $ret[] = sprintf('<label for="edit_%s">%s</label><br>', $id, $info['name']);
      $ret[] = sprintf('<input type="checkbox" id="edit_%s" name="%s">', $id, $id);
    } elseif ($info['type'] == 'select') {
      $v = array('id="edit_'.$id.'"',
		 'name="'.$id.'"');
      if (array_key_exists('onchange',$info))
	$v[] = sprintf('onchange="%s"', $info['onchange']);

      $ret[] = sprintf('<label for="edit_%s">%s</label><br>', $id, $info['name']);
      $ret[] = sprintf('<select %s>', implode(' ', $v));
      if (is_array($info['options'])) $options = $info['options'];
      else $options = query_db($info['options']);
      foreach ($options as $row)
	$ret[] = sprintf('<option value="%s"%s>%s</option>',
			 $row['id'],
			 !empty($info['selected']) && $row['id'] == $info['selected'] ? ' selected' : '',
			 $row['text']);
      $ret[] = '</select>';
    } elseif ($info['type'] == 'textarea') {
      $ret[] = sprintf('<label for="edit_%s">%s</label><br>', $id, $info['name']);
      $v = array('id="edit_'.$id.'"',
		 'name="'.$id.'"');
      $v[] = sprintf('cols="%d"', empty($info['columns'])?33:$info['columns']);
      $v[] = sprintf('rows="%d"', empty($info['rows'])?5:$info['columns']);
      $ret[] = sprintf('<textarea %s></textarea>', implode(' ', $v));
    }
    if (array_key_exists('comment',$info))
      $ret[] = sprintf('<span class="comment">%s</span>', $info['comment']);
    if ($info['type'] != 'hidden')
      $ret[] = '<br>';
  }

  $v_save = array('id="button_save"',
		  'onclick="return form_save(this);"');
  if ($mask['buttons']['save'] === false) $v_save[] = 'style="display: none"';

  $v_insert = array('id="button_insert"',
		  'onclick="return form_insert(this);"');
  if ($mask['buttons']['insert'] === false) $v_insert[] = 'style="display: none"';

  $v_delete = array('id="button_delete"',
		    'onclick="return form_delete(this);"');
  if ($mask['buttons']['delete'] === false) $v_delete[] = 'style="display: none"';

  $ret[] = '<span id="form_status">&nbsp;</span>';
  $ret[] = '<div class="buttons">';
  $ret[] = sprintf('<button %s>%s</button>', implode(' ', $v_save), 'Speichern');
  $ret[] = '<span></span>';
  $ret[] = sprintf('<button %s>%s</button>', implode(' ', $v_insert), 'Hinzufügen');
  $ret[] = '<span></span>';
  $ret[] = sprintf('<button %s>%s</button>', implode(' ', $v_delete), 'Löschen');
  $ret[] = '</div>';

  $ret[] = '</form>';
  $ret[] = '</div>';

  return $ret;
}

function build_grid($name, $mask)
{
  global $jscode;
  $ret = array();

  $opts = array("click: gridDrillDown",
		"menuEvent: 'contextmenu'",
		"highlightElem: 'menuRow'");

  if (array_key_exists('rows', $mask)) $opts[] = 'visibleRows: ' . $mask['rows'];
  if (array_key_exists('sort', $mask)) $opts[] = 'sortCol: ' . $mask['sort'];
  if ($mask['prefetch'] === false) $opts[] = 'prefetch: false';
  # $opts[] = 'frozenColumns: ' . count($mask['list']);
  $opts[] = 'saveColumnInfo: {width: true, filter: true, sort: true}';

  $ret[] = sprintf('<table id="grid_%s" class="ricoLiveGrid">', $name);
  $ret[] = '  <tr>';
  $specs = array();
  $fields = array();
  foreach ($mask['list'] as $field => $data) {
    $ret[] = sprintf('  <th>%s</th>', $data['name']);
    $s = array(sprintf("FieldName: 'input_%s'", $field),
	       sprintf("ColName: 'input_%s'", $data['name']));
    $s = array();
    if ($data['visible'] === false) $s[] = 'visible: false';
    if ($data['width'] > 0) $s[] = 'width: ' . $data['width'];
    if (array_key_exists('type', $data)) $s[] = "type: '" . $data['type'] . "'";
    if (array_key_exists('specs', $data)) $s[] = $data['specs'];
    if (array_key_exists('control', $data)) $s[] = 'control: ' . $data['control'];
    $specs[] = '{' . implode(', ', $s) . '}';

    if (array_key_exists('sql', $data))
      $fields[] = $data['sql'] . ' AS ' . $field;
    else
      $fields[] = $field;
  }

  grid_sql($name, $mask);

  $ret[] = '  </tr>';
  $ret[] = '</table>';

  $opts[] = 'columnSpecs: [' . implode(', ', $specs) . ']';

  if (strstr($opts[count($opts)-1], 'filterUI') !== false) $opts[] = 'FilterLocation: -1';

  $jscode[] = 'var grid;';
  $jscode[] = 'Rico.onLoad( function() {';
  $jscode[] = sprintf("var %s_opts = {\n  %s\n};", $name, implode(",\n  ", $opts));
  $jscode[] = sprintf("grid = new Rico.LiveGrid ('grid_%s', new Rico.Buffer.AjaxSQL('ajax/ricoXMLquery.php'), %s_opts);", $name, $name);
  $jscode[] = "grid.menu = new Rico.GridMenu();";
  $jscode[] = "grid.edit = new Rico.TableEdit(grid);";
  $jscode[] = '});';
  $jscode[] = "Rico.acceptLanguage('de-de,de;q=0.8,en;q=0.5,en-us;q=0.3');";
  $jscode[] = "Rico.loadModule('Effect','Calendar','LiveGridForms','LiveGridAjax','LiveGridMenu');";

  return $ret;
}

function build_details($name, $details)
{
  $ret = array();

  $ret[] = '<div class="box" id="details">';
  $ret[] = sprintf('<h3>%s</h3>', $details['title']);
  if (array_key_exists('subtitle', $details))
    $ret[] = sprintf('<p class="subtitle">%s</p>', $details['subtitle']);
  if (array_key_exists('html',$details)) {
    $ret[] = $details['html'];
  } elseif (array_key_exists('list',$details)) {
    $ret[] = '<ul>';
    foreach ($details['list'] as $name => $info) {
      $ret[] = sprintf('<li>%s: <span id="detail_%s"></span></li>', $info['name'], $name);
    }
    $ret[] = '</ul>';
  }
  $ret[] = '</div>';

  return $ret;
}

function build_select($name, $details)
{
  $ret = array();

  $ret[] = '<div class="form">';
  $ret[] = sprintf('<p class="title">%s</p>', $details['title']);
  $ret[] = sprintf ('<select%s>', array_key_exists('onchange',$details)?sprintf(' onchange="%s"',$details['onchange']):'');
  if (is_array($details['options']))
    $options = $details['options'];
  else
    $options = query_db($details['options']);

  if (array_key_exists('default',$details))
    $ret[] = sprintf('<option value="">%s</option>', $details['default']);

  foreach ($options as $row) {
    $selected = array_key_exists('selected',$details) && $details['selected'] == $row['id'] ? true : false;
    $ret[] = sprintf('<option value="%s"%s>%s</option>', $row['id'], $selected?' selected':'',$row['text']);
  }

  $ret[] = '</select>';
  $ret[] = '</div>';

  return $ret;
}

function build_mask($name, $mask)
{
  global $jscode;
  $jscode[] = 'Rico.writeDebugMsg = function(msg, resetFlag) {};';

  if (array_key_exists('list', $mask))
    $grid = build_grid($name, $mask);
  else
    $grid = array();

  $status = array('<span id="status"></span><br>');

  if (array_key_exists('details', $mask))
    $details = build_details($name, $mask['details']);
  else
    $details = array();

  if (array_key_exists('select', $mask))
    $select = build_select($name, $mask['select']);
  else
    $select = array();

  if (array_key_exists('edit', $mask) && check_edit($name))
    $edit = build_form($name, $mask);
  else
    $edit = array();

  $title = $mask['title'];
  if (array_key_exists('subtitle', $mask))
    $title .= ' &ndash; ' . $mask['subtitle'];

  $head = array();
  $head[] = sprintf('<h3>%s</h3>', $title);

  if (array_key_exists('html', $mask) &&
      !array_key_exists('details', $mask) &&
      !array_key_exists('select', $mask) &&
      !array_key_exists('edit', $mask))
    return array_merge($head, array($mask['html']),$status);

  return array_merge($head,
		     array('<div class="right">'),
		     $grid,
		     array('</div>','<div class="left">'),
		     $details,
		     $select,
		     $edit,
		     $status,
		     array('</div>'));
}

function mask($name)
{
  global $jsfiles;
  global $mask;

  $ret = array();

  if (load_mask($name) === false)
    return;

  $jsfiles[] = 'lib/functions.js';
  $jsfiles[] = 'lib/rico/rico.js';

  if (array_key_exists('table',$mask))
    $ret = build_mask($name, $mask);

  return sprintf('<span id="source" style="display: none;">%s</span>', $_GET['mask'])
    . implode("\n", $ret);
}

?>
